function likelihood = get_likelihood_by_chunking(triplets, related, S, O, U, V, alpha, Y, params)

% This function computes the likelihood for each datapoint in {triplets,related}, based on the model specified by the parameters {S, O, U, V, alpha, Y, params}
%
% To avoid any memory issue in case where the number of datapoints would too large, the dataset is split into chunks processed independently
%
% Copyright (C) 2012 Rodolphe Jenatton, Nicolas Le Roux, Antoine Bordes, Guillaume Obozinski
%         
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


	N = rows(triplets);

	likelihood = zeros(N,1);

	if N <= 1000, 
		chunksize = N;
		n_chunks  = 1;
	else
		chunksize = 1000;
		n_chunks = floor(N/chunksize);
	end

	index = 0;

	for chunk = 1:n_chunks,

		% Be careful for the last batch, it may be bigger than the others.
		if chunk == n_chunks
			list_indices = (index+1) : N;
		else
			list_indices = index + (1:chunksize);
		end
		[dummy, dummy, likelihood(list_indices)] = tensor_factor_evaluation(triplets(list_indices,:), related(list_indices), S, O, U, V, alpha, Y, params, 1);

		index = index + chunksize;
	end

	likelihood = exp(-likelihood);

end
