function neg_triplets = generate_neg_relationships(triplets, n_neg_relationships, sample_type)

% Sample a number n_neg_relationships of negative relationships according to two possible schemes:
%
% sample_type: {'any', 'rel'}
% When sample_type ==  'any', the negative triplets are generated by randomly sampling one of the {subject,relationship,object} while keeping fixed the two others.
% When sample_type ==  'rel', the negative triplets are generated by randomly sampling a relationship while keeping fixed the {subject,object}
%
% Copyright (C) 2012 Rodolphe Jenatton, Nicolas Le Roux, Antoine Bordes, Guillaume Obozinski
%         
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

	max_indices = max(triplets,[],1);


	switch sample_type

	case 'rel'

		neg_triplets = get_neg_triplets_part( triplets, n_neg_relationships, max_indices, 2);

	case 'any'

		neg_triplets = zeros(n_neg_relationships,3);

		n_neg = floor(n_neg_relationships/3);

		neg_triplets(1:n_neg,:)         = get_neg_triplets_part( triplets, n_neg, max_indices, 1);
		neg_triplets(n_neg+(1:n_neg),:) = get_neg_triplets_part( triplets, n_neg, max_indices, 2);
		neg_triplets((2*n_neg+1):end,:) = get_neg_triplets_part( triplets, n_neg_relationships-2*n_neg, max_indices, 3);

	end
end

function neg_triplets_part = get_neg_triplets_part( triplets, n_neg, max_indices, i_col)

	N = max_indices(i_col);
	n_triplets = rows(triplets);

        line_to_consider = floor( n_triplets * rand(n_neg, 1) ) + 1;

        neg_triplets_part = triplets(line_to_consider, :);

        pos_elt = triplets(line_to_consider, i_col);

	neg_indices = floor(( N - 1)*rand(n_neg, 1)) + 1;

	neg_triplets_part(:, i_col) = ...
		neg_indices .* (neg_indices < pos_elt) + ...
		(neg_indices + 1) .* (neg_indices >= pos_elt);
	
end
