function params = ask_params(params, field, type, question, value)

% params = ask_params(params, field, type, question, value) checks if the field
% 'field' exists. If it doesn't, it will ask the user the question 'question'.
% The type allows for different types of answer (strings, binary, real, ...).
% The default value for the parameter is 'value'.
%
% Copyright (C) 2012 Rodolphe Jenatton, Nicolas Le Roux, Antoine Bordes, Guillaume Obozinski
%         
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.


wrong_input = 0;
switch type
case 'pos'
	while ~isfield(params,field) | ~isnumeric(params.(field)) | params.(field) < 0
	 if wrong_input == 1, fprintf('Improper value.\n'); end
	 params.(field) = input(question);
	 if isempty(params.(field)), params.(field) = value; end
	 wrong_input = 1;
	end

case 'pos_int'
	while ~isfield(params,field) | ~isnumeric(params.(field)) | params.(field) < 0 | round(params.(field)) ~= params.(field)
	 if wrong_input == 1, fprintf('Improper value.\n'); end
	 params.(field) = input(question);
	 if isempty(params.(field)), params.(field) = value; end
	 wrong_input = 1;
	end

case 'real'
	while ~isfield(params,field) | ~isnumeric(params.(field))
	 if wrong_input == 1, fprintf('Improper value.\n'); end
	 params.(field) = input(question);
	 if isempty(params.(field)), params.(field) = value; end
	 wrong_input = 1;
	end

case 'binary'
	while ~isfield(params,field) | ~isnumeric(params.(field)) | (params.(field) ~= 0 & params.(field) ~= 1)
	 if wrong_input == 1, fprintf('Improper value.\n'); end
	 params.(field) = input(question);
	 if isempty(params.(field)), params.(field) = value; end
	 wrong_input = 1;
	end

case 'string'
	while ~isfield(params,field) | ~isstr(params.(field))
	 if wrong_input == 1, fprintf('Improper value.\n'); end
	 params.(field) = input(question);
	 if isempty(params.(field)), params.(field) = value; end
	 wrong_input = 1;
	end
end
