function str = sec_to_time(n)

% Takes a time in seconds and returns a string containing the time in proper notations.
%
% Copyright (C) 2012 Rodolphe Jenatton, Nicolas Le Roux, Antoine Bordes, Guillaume Obozinski
%         
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

n = floor(n);

sec_in_minute = 60;
sec_in_hour = 3600;
sec_in_day = 86400;
sec_in_week = 604800;

str = '';

%% weeks %%

n_weeks = floor(n / sec_in_week);
if n_weeks > 0
	str = sprintf('%iw ', n_weeks);
end

n = mod(n, sec_in_week);

%% days %%

n_days = floor(n / sec_in_day);
if (n_weeks > 0 || n_days > 0)
	str = sprintf('%s%id ', str, n_days);
end

n = mod(n, sec_in_day);

%% hours %%

n_hours = floor(n / sec_in_hour);
if (n_weeks > 0 || n_days > 0 || n_hours > 0)
	str = sprintf('%s%ih ', str, n_hours);
end

n = mod(n, sec_in_hour);

%% minutes %%

n_minutes = floor(n / sec_in_minute);
if (n_weeks > 0 || n_days > 0 || n_hours > 0 || n_minutes > 0)
	str = sprintf('%s%im ', str, n_minutes);
end

n = mod(n, sec_in_minute);

%% seconds

str = sprintf('%s%is', str, n);
