//
// Copyright (C) 2012 Rodolphe Jenatton, Nicolas Le Roux, Antoine Bordes, Guillaume Obozinski
//         
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details. 
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include <mexutils.h>
#include <vector>
#include <map>
#include <algorithm>
#include <iostream>

using namespace std;

typedef vector<mwIndex> vec;
typedef vec::const_iterator vec_it;

struct row_col {
    mwIndex row, col;
};

typedef vector<row_col> vec_pair;
typedef vec_pair::const_iterator vec_pair_it;

typedef map< mwIndex, vec >::const_iterator map_vec_it;
typedef map< mwIndex, vec_pair >::const_iterator map_vec_pair_it;

void getMapIDX(double *idx, mwIndex row, mwIndex col, map< mwIndex, vec_pair >& map_IDX, vec& unique_IDX)
{
    mwIndex i, j, k;
    for (j=0;j<col;j++)
        for (i=0;i<row;i++)
        {
            k = idx[i+j*row]-1;
            
            row_col pair;
            
            pair.row = i;
            pair.col = j;
            
            map_IDX[k].push_back(pair);
        }
    
   for (map_vec_pair_it it=map_IDX.begin(); it != map_IDX.end(); it++)
      unique_IDX.push_back(it->first);
  
   sort(unique_IDX.begin(), unique_IDX.end());
}

void SetSpMat(mxArray* p, vec& unique)
{
    const mwSize row = mxGetM (p);
    const mwSize col = mxGetN (p);
    const mwSize nnz = mxGetNzmax(p);
    
    if (nnz == 0) return;
    
    mwIndex *I, *J;
    mwIndex i, k, start, end;
    double * D;
    
    I = (mwIndex*)mxGetIr (p);
    J = (mwIndex*)mxGetJc (p);
    D = (double*)mxGetPr(p);
    
    for (i=0;i<nnz;i++) D[i] = 0.0;
    
     
     // Fill in J
     start = 0;
     J[0]  = 0;
     
     for (k=0;k<unique.size();k++)
     {
         end = unique[k];
         
         for (i=start;i<end;i++) J[i+1] = J[i];
         
         J[end+1] = J[end] + row;
         
         start = end+1;
     }
     for (i=start; i<col; i++) J[i+1] = J[i];
         
     // Fill in I
     for (i=0;i<nnz;i++) I[i] = i % row;
}

double rescale(double weight, mwIndex idx)
{
    if (idx == 0)
        return 1.0;
    else
        return weight;
}


// Argument 0: idx,   whose dim is n x p
//          1: A       whose dim is ra x n x p
//          2: weight  scalar
//          3: ncol_db scalar
//-------------------------------------------------------------------------

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  
  double *A, *idx, *weight, *ncol_db;
  
  mwSize *dimA;
  mwSize n = mxGetM(prhs[0]), p = mxGetN(prhs[0]);

  idx     = (double*)mxGetPr(prhs[0]);
  A       = (double*)mxGetPr(prhs[1]);
  weight  = (double*)mxGetPr(prhs[2]);
  ncol_db = (double*)mxGetPr(prhs[3]);
  
  const mwSize ncol = *ncol_db;
  
  dimA = (mwSize*)mxGetDimensions(prhs[1]);
  //-----------------------------------------------------------------------
  vec uniqueA;
  map< mwIndex, vec_pair > mapA;
  
  getMapIDX(idx, n, p, mapA, uniqueA);
  //-----------------------------------------------------------------------

  plhs[0]= mxCreateSparse (dimA[0], ncol, dimA[0]*uniqueA.size(), mxREAL);
  
  SetSpMat(plhs[0], uniqueA);
  //-----------------------------------------------------------------------
  double* gradA = (double*)mxGetPr(plhs[0]);
  
  mwIndex counter = 0;
  for (vec_it k=uniqueA.begin(); k != uniqueA.end() ; ++k)
  {
       for (vec_pair_it it = mapA[*k].begin(); it != mapA[*k].end(); ++it)
           
              for (mwIndex j=0;j<dimA[0];j++)
                  
                    gradA[j+counter] += A[ j +  it->row * dimA[0]  +  it->col * dimA[0] * dimA[1]  ] * rescale( *weight, it->col);
                  
      counter += dimA[0];    
  }
  
}

