//
// Copyright (C) 2012 Rodolphe Jenatton, Nicolas Le Roux, Antoine Bordes, Guillaume Obozinski
//         
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details. 
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
#include <mexutils.h>


/**************************************************************************
This function makes the computation in C++ of the following matlab code:

grad_alpha = delta * bsxfun(@times, A, weight) .* B + A .* ( u * weight) + B .* (v * weight) + ( r .* s ) * weight
***************************************************************************/

// Argument 0: A,     whose dim is na x n
//          1: B,     whose dim is na x n
//          2: u,     whose dim is na x 1 
//          3: v,     whose dim is na x 1 
//          4: r      whose dim is na x 1
//          5: s      whose dim is na x 1
//          6:weight  whose dim is 1 x n
//          7:delta   scalar
//
//-------------------------------------------------------------------------

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
  
  bool  delta;  
  double *A, *B, *u, *v, *r, *s, *weight, *d;
  
  mwIndex i, j;
  mwSize na = mxGetM(prhs[0]), n = mxGetN(prhs[0]);

  A      = (double*)mxGetPr(prhs[0]);
  B      = (double*)mxGetPr(prhs[1]);
  u      = (double*)mxGetPr(prhs[2]);
  v      = (double*)mxGetPr(prhs[3]);
  r      = (double*)mxGetPr(prhs[4]);
  s      = (double*)mxGetPr(prhs[5]);
  weight = (double*)mxGetPr(prhs[6]);
  d      = (double*)mxGetPr(prhs[7]);
  
  delta = *d;
  //-----------------------------------------------------------------------
  plhs[0]     = mxCreateDoubleMatrix(na,n,mxREAL);
  double* OUT = (double*)mxGetPr(plhs[0]);

  if (delta)
  {
      for (j=0;j<n;j++)
          for (i=0;i<na;i++)
              OUT[i+j*na] =  weight[j] * ( A[i+j*na] * B[i+j*na] +  A[i+j*na] * u[i] + B[i+j*na] * v[i] + r[i] * s[i]);
  }
  else
  {
      for (j=0;j<n;j++)
          for (i=0;i<na;i++)
              OUT[i+j*na] =  weight[j] * ( A[i+j*na] * u[i] + B[i+j*na] * v[i] + r[i] * s[i]);
  }
  
}

