function [S, O, U, V, alpha, Y, params] = tensor_factor_update(S, O, U, V, alpha, Y, grad, params)

% tensor_factor_update updates the parameters with the gradients stored in the structure "grad".
%
% Copyright (C) 2012 Rodolphe Jenatton, Nicolas Le Roux, Antoine Bordes, Guillaume Obozinski
%         
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.



% epsilon is the step-size for this particular update.
% The division by params.opt.epsilon_correction is because we compute the gradient
% of the sum of the costs rather than the average.
epsilon = params.opt.rate / (1 + params.current_iter * params.opt.decr) / params.opt.epsilon_correction;

% In place update of S.
mexInPlace_gradient_update(S, -epsilon*grad.S);
if params.model.unique_rep == 0
    mexInPlace_gradient_update(O, -epsilon*grad.O);
end

mexInPlace_gradient_update(alpha, -epsilon*grad.alpha);

U     = U - epsilon*grad.U;
V     = V - epsilon*grad.V;
Y     = Y - epsilon*grad.Y;

% Project Y to the unit ball.
Y = L2_proj(Y,0);

if params.model.ngram(1) == '0' % There is no uni-gram term
    Y(:,[1 2]) = 0;
end
if params.model.ngram(2) == '0' % There is no bi-gram term
    Y(:,[3 4]) = 0;
end

% The tri-gram term is handled in energy_relation

% If we use the same representation for S and O, we use the same vectors y too.
if params.model.unique_rep == 1, 
    Y(:,2) = Y(:,1);
end

% Projections.
% Only do it every params.opt.proj_skip steps,
if params.opt.skip_step == params.opt.proj_skip

    switch params.model.constraint_for_SO
        case 0, % Project onto L2 ball
            S = L2_proj(S,0);
            if params.model.unique_rep == 0, O = L2_proj(O,0); end
        case 1, % Project onto simplex
            S = L1_proj(S,1,1);
            if params.model.unique_rep == 0, O = L1_proj(O,1,1); end
        case 2, % Project onto L2 ball with pos constraints
            S = L2_proj(S,1);
            if params.model.unique_rep == 0, O = L2_proj(O,1); end
        otherwise
            S = L2_proj(S,0);
            if params.model.unique_rep == 0, O = L2_proj(O,0); end
    end
    
	U = L2_proj(U,0);
	V = L2_proj(V,0);
	
	if params.model.alpha > 0
		alpha(:) = L1_proj(alpha(:), params.model.alpha_scaled, 0);
	elseif params.model.alpha < 0
		alpha = abs(params.model.alpha)*L2_proj(alpha/abs(params.model.alpha),0);
    end
	
	params.opt.skip_step = 1;

else
	
	params.opt.skip_step = params.opt.skip_step + 1;

end

if params.model.unique_rep == 1, O = S; end

end
